unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs,

  OpenGL, ExtCtrls;

const
  GridSize = 63;

type
  TGLCoord = record
    X, Y, Z : glFloat;
  end;

var
  h_DC   : HDC;
  h_RC   : HGLRC;
  DemoStart: Cardinal;
  WaterTexture : glUint;
  Viscosity : glFloat;
  Position : Array[0..GridSize, 0..GridSize] of glFloat;
  Velocity : Array[0..GridSize, 0..GridSize] of glFloat;
  Vertex : Array[0..GridSize, 0..GridSize] of TglCoord;
  Normals:array [0..GridSize, 0..GridSize] of TglCoord;
  xAngle, yAngle : glFloat;
  ElapsedTime : Integer;             
  ElapsedTime2 : Integer;


type
  TForm1 = class(TForm)
    Timer2: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure Timer2Timer(Sender: TObject);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

  function gluBuild2DMipmaps(Target: GLenum; Components, Width, Height: GLint;
    Format, atype: GLenum; Data: Pointer): GLint; stdcall; external glu32;
  procedure glGenTextures(n: GLsizei;
    var textures: GLuint); stdcall; external opengl32;
  procedure glBindTexture(target: GLenum;
    texture: GLuint); stdcall; external opengl32;

procedure DrawWater;
var I, J : Integer;
    VectLength : glFloat;
    Speed : glFloat;
begin
  Speed :=ElapsedTime - ElapsedTime2;
  Speed :=Speed/7;

  // Calculate new velocity
  For I :=2 to GridSize-2 do
    For J :=2 to GridSize-2 do
      Velocity[I, J] := Velocity[I, J] + (Position[I, J] -
              (4*(Position[I-1,J] + Position[I+1,J] + Position[I,J-1] + Position[I,J+1]) +  // left, right, above, below
              Position[I-1,J-1] + Position[I+1,J-1] + Position[I-1,J+1] + Position[I+1,J+1])/25) / 7;  // diagonally across

  // Calculate the new ripple positions
  For I:=2 to GridSize-2 do
    For J:=2 to GridSize-2 do
    Begin
      Position[I, J] := Position[I, J] - Velocity[I,J]*Speed;
      Velocity[I, J] := Velocity[I, J] * Viscosity;
    End;

  // Calculate the new vertex coordinates
  For I :=0 to GridSize do
    For J :=0 to GridSize do
    begin
      Vertex[I, J].X :=(I - GridSize/2)/GridSize*2;
      Vertex[I, J].Y :=(Position[I, J]/Speed / 1024)/GridSize*2;
      Vertex[I, J].Z :=(J - GridSize/2)/GridSize*2;
    end;

  For I :=0 to GridSize do
  begin
    For J :=0 to GridSize do
    begin
      If (I > 0) and (J > 0) and (I < GridSize) and (J < GridSize) then
      begin
        with Normals[I, J] do
        begin
          X := Position[I+1, J] - Position[I-1,J];
          Y := -2048;
          Z := Position[I, J+1] - Position[I, J-1];
          VectLength :=sqrt(x*x + y*y + z*z);
          if VectLength <> 0 then
          begin
            X :=X/VectLength;
            Y :=Y/VectLength;
            Z :=Z/VectLength;
          end;
        end;
      end
      else
      begin
        Normals[I, J].X :=0;
        Normals[I, J].Y :=1;
        Normals[I, J].Z :=0;
      end;
    end;
  end;
  glBindTexture(GL_TEXTURE_2D, WaterTexture);
  For J :=0 to GridSize-1 do
  begin
    glBegin(GL_QUAD_STRIP);
      for I :=0 to GridSize do
      begin
        glNormal3fv(@Normals[I, J+1]);
        glVertex3fv(@Vertex[I, J+1]);
        glNormal3fv(@Normals[I, J]);
        glVertex3fv(@Vertex[I, J]);
      end;
    glEnd;
  end;
  ElapsedTime2 :=ElapsedTime;
end;

procedure glDraw;
begin
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
  glLoadIdentity;
  glTranslatef(0.0,0.0,-3);
  glRotatef(xAngle, 1, 0, 0);
  glRotatef(yAngle, 0, 1, 0);
  glColor3f(0.85, 1, 0.85);
  glEnable(GL_BLEND);
  glEnable(GL_TEXTURE_GEN_S);
  glEnable(GL_TEXTURE_GEN_T);
  DrawWater;
end;

procedure LoadTexture(Filename: String; var Texture: GLuint);

  procedure SwapRGB(data : Pointer; Size : Integer);
  asm
    mov ebx, eax
    mov ecx, size
  @@loop :
    mov al,[ebx+0]
    mov ah,[ebx+2]
    mov [ebx+2],al
    mov [ebx+0],ah
    add ebx,3
    dec ecx
    jnz @@loop
  end;

  procedure LoadBitmap(Filename: String; out Width: Cardinal; out Height: Cardinal; out pData: Pointer);
  var
    FileHeader: TBITMAPFILEHEADER;
    InfoHeader: TBITMAPINFOHEADER;
    Palette: array of RGBQUAD;
    BitmapFile: THandle;
    BitmapLength: LongWord;
    PaletteLength: LongWord;
    ReadBytes: LongWord;
  begin
    BitmapFile := CreateFile(PChar(Filename), GENERIC_READ, FILE_SHARE_READ, nil, OPEN_EXISTING, 0, 0);
    if (BitmapFile = INVALID_HANDLE_VALUE) then begin
      MessageBox(0, PChar('Error opening "' + Filename), PChar('BMP Unit'), MB_OK);
      Exit;
    end;
    ReadFile(BitmapFile, FileHeader, SizeOf(FileHeader), ReadBytes, nil);
    ReadFile(BitmapFile, InfoHeader, SizeOf(InfoHeader), ReadBytes, nil);
    PaletteLength := InfoHeader.biClrUsed;
    SetLength(Palette, PaletteLength);
    ReadFile(BitmapFile, Palette, PaletteLength, ReadBytes, nil);
    if (ReadBytes <> PaletteLength) then begin
      MessageBox(0, PChar('Error reading palette'), PChar('BMP Unit'), MB_OK);
      Exit;
    end;
    Width := InfoHeader.biWidth;
    Height := InfoHeader.biHeight;
    BitmapLength := InfoHeader.biSizeImage;
    if BitmapLength = 0 then
      BitmapLength := Width * Height * InfoHeader.biBitCount Div 8;
    GetMem(pData, BitmapLength);
    ReadFile(BitmapFile, pData^, BitmapLength, ReadBytes, nil);
    if (ReadBytes <> BitmapLength) then begin
      MessageBox(0, PChar('Error reading bitmap data'), PChar('BMP Unit'), MB_OK);
      Exit;
    end;
    CloseHandle(BitmapFile);
    SwapRGB(pData, Width*Height);
  end;

var
  pData: Pointer;
  Width: LongWord;
  Height: LongWord;
begin
  pData :=nil;
  LoadBitmap(Filename, Width, Height, pData);
  if pData = nil then
  begin
    MessageBox(0, PChar('Unable to load ' + filename), 'Loading Textures', MB_OK);
    Halt(1);
  end;
  glGenTextures(1, Texture);
  glBindTexture(GL_TEXTURE_2D, Texture);
  glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  gluBuild2DMipmaps(GL_TEXTURE_2D, 3, Width, Height, GL_RGB, GL_UNSIGNED_BYTE, pData);
end;

procedure glInit();
var I, J : Integer;
begin
  glClearColor(0.0, 0.0, 0.0, 0.0); 
  glShadeModel(GL_SMOOTH);          
  glClearDepth(1.0);                
  glEnable(GL_DEPTH_TEST);          
  glDepthFunc(GL_LESS);
  glBlendFunc(GL_SRC_COLOR, GL_ONE);
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
  glEnable(GL_TEXTURE_2D);
  LoadTexture('picture.bmp', WaterTexture);
  glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
  glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
  Viscosity :=0.96;
  For I :=0 to GridSize do
  begin
    For J :=0 to GridSize do
    begin
      Position[I, J] :=0;
      Velocity[I, J] :=0;
    end;
  end;
  Randomize;
  xAngle :=90;
  yAngle :=90;
end;

procedure glResizeWnd(Width, Height : Integer);
begin
  if (Height = 0) then Height := 1;
  glViewport(0, 0, Width, Height);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity;
  gluPerspective(45.0, Width/Height, 1.0, 100.0);
  glMatrixMode(GL_MODELVIEW);         
  glLoadIdentity;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  pfd : TPIXELFORMATDESCRIPTOR;
  PixelFormat : GLuint;
begin
  SetWindowLong(Handle, GWL_STYLE,
    GetWindowLong(Handle, GWL_STYLE) or CS_OWNDC);
  h_DC := GetDC(Handle);
  if (h_DC <> 0) then
  begin
    ZeroMemory(@pfd, SizeOf(TPIXELFORMATDESCRIPTOR));
    with pfd do
    begin
      nSize           := SizeOf(TPIXELFORMATDESCRIPTOR);
      nVersion        := 1;                    
      dwFlags         := PFD_DRAW_TO_WINDOW    
                         or PFD_SUPPORT_OPENGL 
                         or PFD_DOUBLEBUFFER;  
      cColorBits      := 32;
      cDepthBits      := 16;
    end;
    PixelFormat := ChoosePixelFormat(h_DC, @pfd);
    if PixelFormat <> 0 then
      if SetPixelFormat(h_DC, PixelFormat, @pfd) then
      begin
        h_RC := wglCreateContext(h_DC);
        if h_RC <> 0 then
          if wglMakeCurrent(h_DC, h_RC) then
          begin
            glResizeWnd(Width, Height);
            glInit;
          end;
      end;
  end;
  DemoStart := GetTickCount;
end;

procedure TForm1.FormPaint(Sender: TObject);
var
  LastTime : DWord;
begin
  LastTime :=ElapsedTime;
  ElapsedTime :=GetTickCount() - DemoStart;
  ElapsedTime :=(LastTime + ElapsedTime) DIV 2;
  glDraw;
  SwapBuffers(h_DC);
end;

procedure TForm1.Timer2Timer(Sender: TObject);
var
  LastTime : DWord;
begin
  LastTime :=ElapsedTime;
  ElapsedTime :=GetTickCount() - DemoStart;
  ElapsedTime :=(LastTime + ElapsedTime) DIV 2;
  glDraw;
  SwapBuffers(h_DC);
end;

procedure TForm1.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  Velocity[GridSize - MulDiv(Y, GridSize, Height), MulDiv(X, GridSize, Width)] :=1060;
end;

end.
